{% extends "base.html" %}

{% block content %}
<div class="container-fluid">
    <div class="row">
        <div class="col-sm-4">
            <div class="card" style="min-height:350px">
                <div class="card-header">
                    <span id="nodeInfo">Node Info</span>
                </div>
                <div class="card-body">
                    <b>Hardware:</b> {{ nodeInfo['user']['hwModel'] | safe }}<br>
                    <b>Firmware:</b> {{ info['firmware_version'] | safe }}<br>
                    <b>Region:</b> {{ prefs['region'] | safe }}<br>
                    <b>nodeId:</b> {{ nodeInfo['user']['id'] | safe }}<br>
                    <b>longName:</b> {{ nodeInfo['user']['longName'] | safe }}<br>
                    <b>shortName:</b> {{ nodeInfo['user']['shortName'] | safe }}<br>
                </div>
            </div>
        </div>
        <div class="col-sm-4">
            <div class="card" style="min-height:350px">
                <div class="card-header">
                    <span id="nodeInfo">Statistics</span>
                </div>
                <div class="card-body">
                    TBD
                </div>
            </div>
        </div>
        <div class="col-sm-4">
            <div class="card" style="min-height:350px">
                <div class="card-header">
                    <span id="nodeInfo">TBS</span>
                </div>
                <div class="card-body">
                    TBD
                </div>
            </div>
        </div>
    </div>
</div>
{% endblock %}